#!/usr/bin/perl -w 
#	App.pm
#
#	The application object, sort of like a surface
#
#	David J. Goehrig Copyright (C) 2000

package SDL::App;
use strict;
use vars qw(@ISA);

use SDL::sdlpl;

use SDL::Surface;
@ISA = qw(SDL::Surface Exporter);


BEGIN {
	use Exporter();
	use vars qw(@EXPORT);
	@EXPORT = qw(&SDL_SWSURFACE &SDL_HWSURFACE &SDL_ANYFORMAT 
		     &SDL_HWPALETTE &SDL_DOUBLEBUF &SDL_FULLSCREEN
		     &SDL_TEXTWIDTH 
		     &SDL_RESIZABLE	
		     &SDL_HWACCEL	
		     &SDL_SRCCOLORKEY	
		     &SDL_RLEACCELOK	
		     &SDL_RLEACCEL	
		     &SDL_SRCALPHA	
		     &SDL_SRCCLIPPING	 		     
		     &SDL_ASYNCBLIT
		    );
}

sub SDL_HWACCEL    
  { 
   return SDL::sdlpl::sdl_hwaccel();
  }
sub SDL_ASYNCBLIT
  { 
   return SDL::sdlpl::sdl_asyncblit();
  }



#
# SDL_TEXTWIDTH Fri May 26 11:13:04 EDT 2000
#
# I added this function so that one can get the width of a string
#

sub SDL_TEXTWIDTH {
	return SDL::sdlpl::sdl_sfont_text_width(join('',@_));
}

#
# Constants
#

sub SDL_SWSURFACE {
	return SDL::sdlpl::sdl_swsurface();
}

sub SDL_HWSURFACE {
	return SDL::sdlpl::sdl_hwsurface();
}

sub SDL_ANYFORMAT {
	return SDL::sdlpl::sdl_anyformat();
}

sub SDL_HWPALETTE {
	return SDL::sdlpl::sdl_hwpalette();
}

sub SDL_DOUBLEBUF {
	return SDL::sdlpl::sdl_doublebuf();
}

sub SDL_FULLSCREEN {
	return SDL::sdlpl::sdl_fullscreen();
}

#
# App Constructor / Destructor
#

sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my %options = @_;

	SDL::sdlpl::sdl_init(SDL::sdlpl::sdl_init_video()|
			SDL::sdlpl::sdl_init_audio()|
			SDL::sdlpl::sdl_init_cdrom());

	my $t = $options{-title} || "";
	my $it = $options{-icon_title} || $t;
	my $ic = $options{-icon} || "";
	my $w = $options{-width} ||0;
	my $h = $options{-height} || 0;
	my $d = $options{-depth} || 0;
	my $f = $options{-flags} || SDL_ANYFORMAT;

	my $self = new SDL::Surface ();
	SDL::sdlpl::sdl_free_surface($self->{-surface});
	($self->{-surface} = SDL::sdlpl::sdl_set_video_mode($w,$h,$d,$f)) or die SDL::sdlpl::sdl_get_error();
	
	if ($ic and -e $ic)
	  {
	   my $icon = new SDL::Surface -name => $ic;
	   SDL::sdlpl::sdl_wm_set_icon($icon->{-surface});	   
	  }
	SDL::sdlpl::sdl_wm_set_caption($t,$it);
	

	bless $self,$class;
	return $self;
}	

#
# Set title bar & icon title
#

sub title {
	my $self = shift;
	my ($title,$icon);
	if (@_) { $title = shift; 
		if (@_) { $icon = shift; } else { $icon = $title }
		SDL::sdlpl::sdl_wm_set_caption($title,$icon);
	}
	return SDL::sdlpl::sdl_wm_get_caption();
}

#
# Set a delay
#

sub delay {
	my $self = shift;
	my $delay = shift;
	SDL::sdlpl::sdl_delay($delay);
}

#
# Get ticks from start
#

sub ticks {
	return SDL::sdlpl::sdl_get_ticks();
}

#
# Get pending error messages if any
#
	
sub error {
	return SDL::sdlpl::sdl_get_error();
}

#
# Pointer warping
#

sub warp {
	my $self = shift;
	my $x = shift;
	my $y = shift;
	SDL::sdlpl::sdl_warp_mouse($x,$y);
}


1;

__END__;

=head1 NAME

SDL::App - a SDL perl extension

=head1 SYNOPSIS

  $app = new SDL::App ( -title => 'FunkMeister 2000', 
			-icon_title => 'FM2000',
			-icon => 'funkmeister.png', 
			-width => 400, 
			-height => 400 );

=head1 DESCRIPTION


This Object is a composite made up of a few surfaces,
and strings.  This object does the setup for the SDL
library, and greates the base application window.  

The options that the constructor takes are:
	
-title =>	the title bar of the window
-icon_title =>	the title bar of the icon
-icon =>	the icon image file
-flags =>	the SDL_* surface flags for the	default window
-width =>	of the app window
-height =>	of the app window
-depth =>	the bit depth of the window

=head2 Additional Methods

	In addition to initilizing the SDL library, the app class
provides access to a couple miscellaneous functions that all programs
may need:

	$app->delay(milliseconds);

Delay will cause a delay of roughly the specified milliseconds. Since
this only runs in multi-tasking environments, the exact number is not
assured.

	$app->ticks();

Ticks returns the number of clock ticks since the program was started.

	$app->error();

Error returns any pending SDL related error messages.  It returns an
empty string if no errors are pending, making it print friendly.

	$app->warp(x,y);

Will move the cursor to the location x,y.

=head1 AUTHOR

David J. Goehrig

=head1 SEE ALSO

perl(1) SDL::Surface(3) SDL::Mixer(3) SDL::Event(3) SDL::Cdrom(3).

=cut	

